package es.us.eii.pfc.jcrbib.data;

import static es.us.eii.pfc.jcrbib.utils.ConstantsUtils.LOGGERNAME;

import java.util.SortedSet;
import java.util.TreeSet;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.apache.log4j.Logger;

import es.us.eii.pfc.jcrbib.domain.Tag;
import es.us.eii.pfc.jcrbib.domain.ex.InvalidParameterException;
import es.us.eii.pfc.jcrbib.domain.ex.ItemAlreadyExistsException;
import es.us.eii.pfc.jcrbib.domain.ex.ItemNotExistsException;
import es.us.eii.pfc.jcrbib.utils.NodeUtils;
import es.us.eii.pfc.jcrbib.utils.RepositoryUtils;
import es.us.eii.pfc.jcrbib.utils.SessionUtils;

/**
 * Clase que implementa el almacenamiento y consulta de categoras de usuario
 * 
 * 
 * 
 */
public class JcrTagDAO implements ITagDAO {

	private static Logger log = Logger.getLogger(LOGGERNAME);

	private static String WEIGHT_PROPERTY = MessagesDAO
			.getString("JcrTagDAO.weight_name_tag");

	private static String TAGS_NODE_NAME = MessagesDAO
			.getString("JcrTagDAO.tags_name_tag"); //$NON-NLS-1$

	protected JcrTagDAO() {

	}

	/**
	 * Mtodo para aadir categorias al sistema. Nunca es invocado de forma
	 * independiente
	 * 
	 * @param tag
	 *            Nombre de la categora a aadir
	 * @param s
	 *            Sesin para realizar los procesos
	 * @throws ItemAlreadyExistsException
	 *             Si la etiqueta ya exista
	 */
	protected void addTag(Tag tag, Session s) throws ItemAlreadyExistsException {
		Node init;
		try {
			log
					.debug(MessagesDAO
							.getString("JcrTagDAO.messages_inserting_tag")); //$NON-NLS-1$
			init = RepositoryUtils.getInitNode(s);
			Node tags = init.getNode(TAGS_NODE_NAME);
			// Codificamos el nombre de la etiqueta para almacenarla
			String encTagName = NodeUtils.encodeTagName(tag.getName());

			// Si no existe
			if (!tags.hasNode(encTagName)) {
				Node tagNode = tags.addNode(encTagName);
				tagNode.setProperty(WEIGHT_PROPERTY, (long) tag.getWeight()); //$NON-NLS-1$
			} else {
				// En otro caso, elevamos la excepcin
				throw new ItemAlreadyExistsException("Tag " + tag.getName()
						+ " already exists");
			}

			// Salvamos los cambios
			s.save();
			log.debug(MessagesDAO
					.getString("JcrTagDAO.messages_succesfully_inserted_tag")); //$NON-NLS-1$
		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.ITagDAO#existsTag(java.lang.String)
	 */
	public boolean existsTag(String name) {
		boolean existsTag = false;
		Session s = SessionUtils.getSession();
		existsTag = existsTag(name, s);
		s.logout();
		log.debug("Cerrando session");
		return existsTag;
	}

	/*
	 * (non-Javadoc) Mtodo que evala si existe una categora
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.JcrTagDAO#existsTag(java.lang.String)
	 */
	protected boolean existsTag(String name, Session s) {
		boolean existsTag = false;
		Node init;
		try {
			// Obtenemos el nodo inicial
			init = RepositoryUtils.getInitNode(s);

			// Obtenemos el nodo de las categoras
			Node tags = init.getNode(TAGS_NODE_NAME);

			// Codificamos el nombre de la categora
			String encTagName = NodeUtils.encodeTagName(name);

			// y preguntamos al repositorio si existe
			existsTag = tags.hasNode(encTagName);
			log
					.debug(MessagesDAO
							.getString("JcrTagDAO.messages_exists_tag_first") + name + MessagesDAO.getString("JcrTagDAO.messages_exists_tag_last") + existsTag); //$NON-NLS-1$ //$NON-NLS-2$
		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		}

		return existsTag;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.ITagDAO#getTag(java.lang.String)
	 */
	public Tag getTag(String name) {
		Tag tag = null;
		Session s = SessionUtils.getSession();
		tag = getTag(name, s);
		s.logout();
		log.debug("Cerrando session");
		return tag;
	}

	/*
	 * (non-Javadoc) Mtodo que obtiene una categora del sistema
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.JcrTagDAO#getTag(java.lang.String)
	 */
	protected Tag getTag(String name, Session s) {
		Node init;
		Tag tag = null;
		try {
			init = RepositoryUtils.getInitNode(s);
			Node tags = init.getNode(TAGS_NODE_NAME);
			String encTagName = NodeUtils.encodeTagName(name);

			// Si existe la categora
			if (tags.hasNode(encTagName)) {
				Node tagNode = tags.getNode(encTagName);

				// Creamos el objeto Tag
				tag = new Tag(name);

				// Completamos sus propiedades
				tag.setWeight((int) tagNode.getProperty(WEIGHT_PROPERTY)
						.getValue() //$NON-NLS-1$
						.getLong());
			}
		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		}
		return tag;
	}

	/**
	 * Establece un nuevo peso para una categora
	 * 
	 * @param tagName
	 *            Categora a alterar
	 * @param newWeight
	 *            Nuevo peso
	 * @param s
	 *            Sesin a usar
	 * @throws InvalidParameterException
	 *             Si el nuevo peso tiene un valor >=0
	 * @throws ItemNotExistsException
	 *             Si la categora no existe
	 */
	protected void setTagWeight(String tagName, int newWeight, Session s)
			throws InvalidParameterException, ItemNotExistsException {
		Node init;

		// Si el peso no es vlido, se eleva una excepcin
		if (newWeight < 0) {
			throw new InvalidParameterException("Weight value:" + newWeight
					+ " must be zero or more");
		}
		try {
			init = RepositoryUtils.getInitNode(s);
			Node tagsNode = init.getNode(TAGS_NODE_NAME);
			String encTagName = NodeUtils.encodeTagName(tagName);

			// Si existe la etiqueta
			if (tagsNode.hasNode(encTagName)) {
				Node tagAct = tagsNode.getNode(encTagName);
				tagAct.setProperty(WEIGHT_PROPERTY, newWeight);
			} else {
				throw new ItemNotExistsException("Tag " + tagName
						+ " not exists");
			}
			s.save();
		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		}
	}

	protected int getTagWeight(String tagName, Session s) {
		int tagWeight = -1;
		Node init;
		try {
			init = RepositoryUtils.getInitNode(s);
			Node tags = init.getNode(TAGS_NODE_NAME);
			String encTagName = NodeUtils.encodeTagName(tagName);
			if (tags.hasNode(encTagName)) {
				Node tagNode = tags.getNode(encTagName);

				// Modificamos su peso
				tagWeight = (int) tagNode.getProperty(WEIGHT_PROPERTY)
						.getValue().getLong();
			}
		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		}
		return tagWeight;
	}

	/*
	 * (non-Javadoc) Mtodo que obtiene las etiquetas del sistema
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.JcrTagDAO#getTags()
	 */
	protected SortedSet<Tag> getTags(Session s) {

		// Conjunto donde almacenaremos el resultado
		SortedSet<Tag> outTags = new TreeSet<Tag>();
		Node init;
		try {
			init = RepositoryUtils.getInitNode(s);
			Node tagsNode = init.getNode(TAGS_NODE_NAME);

			// Iteramos sobre las categoras
			NodeIterator nt = tagsNode.getNodes();
			Tag tagAct;
			Node tagNodeAct;
			while (nt.hasNext()) {
				tagNodeAct = nt.nextNode();

				tagAct = getTag(NodeUtils.decodeTagName(tagNodeAct.getName()),
						s);
				// Vamos aadiendo cada una
				outTags.add(tagAct);
			}
		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		}

		return outTags;
	}

	/**
	 * Mtodo que elimina una categora del sistema
	 * 
	 * @param tagName
	 *            Categora a eliminar
	 * @param s
	 *            Sesin a usar
	 * @throws ItemNotExistsException
	 *             Si la categora no existe
	 */
	protected void removeTag(String tagName, Session s)
			throws ItemNotExistsException {
		Node init;
		try {
			init = RepositoryUtils.getInitNode(s);
			Node tagsNode = init.getNode(TAGS_NODE_NAME);
			String encTagName = NodeUtils.encodeTagName(tagName);
			// Si la categora existe
			if (tagsNode.hasNode(encTagName)) {
				// Eliminamos el nodo (categora)
				tagsNode.getNode(encTagName).remove();
			} else {
				// En otro caso, elevamos una excepcin
				throw new ItemNotExistsException("Tag " + tagName
						+ " not exists");
			}
			// Salvamos los cambios
			s.save();
		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		}

	}

	/**
	 * Mtodo usado para actualizar una categora
	 * 
	 * @param tag
	 *            Categora a actualizar
	 * @param s
	 *            Sesin a usar
	 * @return La categora actualizada
	 * @throws ItemNotExistsException
	 *             Si la categora no exista en el sistema
	 */
	protected Tag updateTag(Tag tag, Session s) throws ItemNotExistsException {
		Tag myTag = null;
		if (existsTag(tag.getName(), s)) {
			myTag = getTag(tag.getName(), s);
			myTag.setWeight(tag.getWeight());
			Node init;
			try {
				init = RepositoryUtils.getInitNode(s);
				Node tags = init.getNode(TAGS_NODE_NAME);
				String encTagName = NodeUtils.encodeTagName(tag.getName());
				Node updateTagNode = tags.getNode(encTagName);
				updateTagNode.setProperty(WEIGHT_PROPERTY, tag.getWeight()); //$NON-NLS-1$
				s.save();
			} catch (PathNotFoundException e) {
				log.error(e);
			} catch (RepositoryException e) {
				log.error(e);
			}
		} else {
			throw new ItemNotExistsException("Tag " + tag.getName()
					+ " don't exists");
		}
		return myTag;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.ITagDAO#getTags()
	 */
	public SortedSet<Tag> getTags() {
		Session s = SessionUtils.getSession();
		SortedSet<Tag> tags;
		tags = getTags(s);
		s.logout();
		log.debug("Cerrando session");
		return tags;
	}
}
